import time
import random
import math
timer_list = {}


def create_timer(id: str = None):
    if id is None:
        id = str(random.randint(int(1e7), int(1e8)-1))

    if id in timer_list:
        return timer_list[id]
    t = Timer()
    t.id = id
    timer_list[id] = t
    return t


class Timer:
    def __init__(self) -> None:
        self.progress_total = 0
        self._progress_current = 0
        self.progress_last = 0
        self.start()

    def start(self) -> float:
        self.time_start = time.time()
        return self.time_start

    @property
    def progress_current(self):
        return self._progress_current

    @progress_current.setter
    def progress_current(self, value):
        self.progress_last = self.progress_current
        self._progress_current = value

    @property
    def spent(self) -> float:
        return time.time() - self.time_start

    @property
    def progress(self) -> float:
        if self.progress_total <= 0:
            return 0
        return self.progress_current / self.progress_total

    @property
    def progress_description(self) -> str:
        '''
        get description with format `progress%`
        '''
        p = self.progress * 100
        v = '{:.2f}'.format(p)
        return f'{v}%'

    @property
    def progress_description_with_value(self) -> str:
        '''
        get description with format `value/total(progress%)`
        '''
        d = self.progress_description
        return f'{self.progress_current}/{self.progress_total}({self.progress_description})'

    @property
    def left_time(self) -> float:
        '''
        返回剩余时间/秒
        '''
        p = self.progress
        if p <= 0:
            return 0
        spent = self.spent
        result = spent / p - spent
        return math.ceil(result * 100) / 100



import os
import random
from sgtpyutils import configuration


def test_primary():
    r = random.randint(int(1e7), int(1e8)-1)
    configuration.configuration_path = f'.tmp.{r}.conf'
    configuration.save({
        'test': 1
    })
    p = configuration.get_config_path()
    assert os.path.exists(p)
    with open(p, 'r', encoding='utf-8') as f:
        data = f.read()
        assert data == '{"test": 1}'

    config = configuration.load()
    assert 'test' in config
    assert config['test'] == 1
    os.remove(p)


def test_set_get():
    r = random.randint(int(1e7), int(1e8)-1)
    configuration.load(config_path=f'.tmp.{r}.conf')
    p = configuration.get_config_path()
    configuration.set('test', 1)
    assert configuration.get('test', 1)
    configuration.set('test2', '1')
    assert configuration.get('test2', '1')
    os.remove(p)



from Crypto.Cipher import AES, DES, DES3
from .utils import padding_to


def main(key: bytes, iv: bytes, content: bytes):
    # MODE_CBC
    e = AES.new(padding_to(key, 32, 32), AES.MODE_CBC, padding_to(
        iv, 16, 16)).encrypt(padding_to(content, 16))
    d = AES.new(padding_to(key, 32, 32), AES.MODE_CBC,
                padding_to(iv, 16, 16)).decrypt(e)
    print(f'e = {e}\nd = {d}\n\n')
    # MODE_CFB
    e = AES.new(padding_to(key, 32, 32), AES.MODE_CFB, padding_to(
        iv, 16, 16)).encrypt(padding_to(content, 16))
    d = AES.new(padding_to(key, 32, 32), AES.MODE_CFB,
                padding_to(iv, 16, 16)).decrypt(e)
    print(f'e = {e}\nd = {d}\n\n')
    # MODE_ECB no need for iv
    e = AES.new(padding_to(key, 32, 32), AES.MODE_ECB).encrypt(
        padding_to(content, 16))
    d = AES.new(padding_to(key, 32, 32), AES.MODE_ECB).decrypt(e)
    print(f'e = {e}\nd = {d}\n\n')
    # MODE_OFB
    e = AES.new(padding_to(key, 32, 32), AES.MODE_OFB, padding_to(
        iv, 16, 16)).encrypt(padding_to(content, 16))
    d = AES.new(padding_to(key, 32, 32), AES.MODE_OFB,
                padding_to(iv, 16, 16)).decrypt(e)
    print(f'e = {e}\nd = {d}\n\n')



from sgtpyutils.binary_tree import BinaryTree



def test_order():
    t = BinaryTree.from_full_tree('123456')
    '124536' == t.travel_preorder()
    '425163' == t.travel_inorder()
    '452631' == t.travel_postorder()


def test_full_tree():
    raw = 'icnerrseaetrvee'
    t = BinaryTree.from_full_tree_preorder(raw)
    result = t.to_list()
    assert 'icanreversetree' == result


def test_recover():
    post_order = '20f0Th{2tsIS_icArE}e7__w'
    in_order = '2f0t02T{hcsiI_SwA__r7Ee}'
    t = BinaryTree.from_postorder_and_inorder(post_order, in_order)
    result = t.travel_preorder()
    assert 'wctf2020{This_IS_A_7reE}' == result



from sgtpyutils.encode import basexx


def test_base64():
    content = 'UOIzVDP2Vzs5UDEyUzL1Vvc4WOApUvU0VOY3WDwoUOIz'
    problem_table = 'ABCDEFGHIJSTUVWKLMNOPQRXYZabcdqrstuvwxefghijklmnopyz0123456789+/'
    result = basexx.base64_decode(content=content, problem_table=problem_table)
    assert result == b'123456789012345678901234567890123'

    result = basexx.base64_encode(content=result, problem_table=problem_table)
    assert result == content


def test_base32():
    content = 'TRLT5amLBoLT5Z6Fa5LqN6mkTomqR66Da4LqX5mgBwkkP5wmTZ6D===='
    problem_table = 'NoPqRsTuVwXyZaBcDeFgHiJkLm765432'
    result = basexx.base32_decode(content=content, problem_table=problem_table)
    assert result == b'10n78ppn3ro00o70r2opop5s3roqq937'

    result = basexx.base32_encode(content=result, problem_table=problem_table)
    assert result == content


def test_base16():
    content = b'testinfo'
    result = basexx.base16_encode(content=content)
    assert '74657374696E666F' == result

    result = basexx.base16_decode(content=result)
    assert content == result


def test_base58():
    content = b'testinfo'
    result = basexx.base58_encode(content=content)
    assert 'LUC1e9BXj9g' == result

    result = basexx.base58_decode(content=result)
    assert content == result


def test_base85():
    content = b'testinfo'
    result = basexx.base85_encode(content=content)
    assert 'bY*jNX>Mk3' == result

    result = basexx.base85_decode(content=result)
    assert content == result


def test_base91():
    content = b'testinfo'
    result = basexx.base91_encode(content=content)
    assert 'fPNKP:Qk1T' == result

    result = basexx.base91_decode(content=result)
    assert content == result


import sgtpyutils.extensions
import sgtpyutils.extensions.clazz
from sgtpyutils.extensions.itertools import run_cycle
from sgtpyutils.hash import HashAlgo, get_hash


def test_fields():
    class TestA:
        def __init__(self) -> None:
            self.A1 = 0
            self.A2 = '0'

    class TestB(TestA):
        def __init__(self) -> None:
            super().__init__()
            self.B1 = '1'
    fields = sgtpyutils.extensions.clazz.get_fields(TestB(), with_parent=False)
    assert 'A1' not in fields
    assert 'B1' in fields
    assert fields['B1'] == '1'
    fields = sgtpyutils.extensions.clazz.get_fields(TestB(), with_parent=True)
    assert 'A1' in fields
    assert 'B1' in fields
    assert fields['A1'] == 0

def test_flat():
    flat = sgtpyutils.extensions.flat
    items = ['1', '2', '3', '4', '5']
    assert items == flat(['1', ['2', '3'], '4', ['5']])
    assert items == flat([[['1']], ['2', ['3', '4'], '5']], 2)
    assert items == flat([[['1']], ['2', ['3', ['4']], '5']])


def test_find():
    find = sgtpyutils.extensions.find
    items = ['1', '2', '3', '4', '5']
    assert '3' == find(items, lambda x: x == '3')
    assert '3' == find(items, lambda x: int(x) == 3)
    assert '3' == find(items, lambda x, index: index == 2)
    assert None == find(items, lambda x, index: index == 7)


def test_convert_bytes():
    convert = sgtpyutils.extensions.cast2bytes
    assert convert(b'123') == b'123'
    assert convert('123') == b'123'
    assert convert((1 << 8)+(2 << 4)+(3 << 0)) == b'\x01\x23'
    assert convert(['1', '2', '3']) == b'123'
    assert convert(['12', '3']) == b'123'
    assert convert(['1', [[['2']], '3']]) == b'123'
    assert convert(['测', [[['试']], '3']]) == '测试3'.encode('utf-8')


def test_xor():
    xor = sgtpyutils.extensions.xor
    assert xor(1, 0) == b'\x01'
    assert xor(
        123, 321) == b'\x7a', '123 ^ 321 = \x7b ^ 0x0141 = \x7b ^ \0x01 == \x7a'
    assert xor('123', '123') == b'\x00\x00\x00'
    key = bytearray.fromhex('057D411526')
    v = b'~!3a@'
    assert xor(b'{\\rtf', key) == v
    assert xor([b'{\\', b'rtf', ], key) == v


def test_cycle():
    a = run_cycle('123abc456', 10)
    result = [''.join(x[1]) for x in a]
    assert len(result) == 10
    assert result == ['11', '21', '31', 'a1',
                      'b1', 'c1', '41', '51', '61', '12']

    import string
    a = run_cycle(string.digits, 10)
    result = [''.join(x[1]) for x in a]
    assert len(result) == 10
    assert result == ['0', '1', '2', '3', '4', '5', '6', '7', '8', '9']

    a = run_cycle(string.digits, length=1)
    result = [''.join(x[1]) for x in a]
    assert len(result) == 10
    assert result == ['0', '1', '2', '3', '4', '5', '6', '7', '8', '9']

    a = run_cycle(string.digits, length=2)
    result = [''.join(x[1]) for x in a]
    assert len(result) == 100
    t = ['0', '1', '2', '3', '4', '5', '6', '7', '8', '9']
    r = []
    for i in range(10):
        r += [f'{x}{i}' for x in t]
    assert result == r


def test_hash():
    get_hash('123456', HashAlgo.md5) == 'e10adc3949ba59abbe56e057f20f883e'
    get_hash('123456', HashAlgo.sha1) == '7c4a8d09ca3762af61e59520943dc26494f8941b'

def test_dict2obj():
    class X:
        def __init__(self) -> None:
            self.a = 0
    x = {'a':1}

    dict2obj = sgtpyutils.extensions.clazz.dict2obj
    x:X = dict2obj(X(),x)
    assert x.a == 1
def test_list2dict():
    a = [2,4,7]
    list2dict = sgtpyutils.extensions.list2dict
    x = list2dict(a)
    assert x.get(0) == 2
    assert x.get(1) == 4
    assert x.get(2) == 7


from sgtpyutils.timer import create_timer, Timer
import time


def test_timer_start():
    t = create_timer()
    assert not t.id is None


def test_time_counter():
    t = create_timer()
    time.sleep(1)
    assert t.spent > 0.5


def test_time_progrss():
    t = create_timer()
    t.progress_total = 100
    time.sleep(1)
    t.progress_current = 50
    assert t.progress_last == 0
    assert t.left_time > 0.8 and t.left_time < 2

    time.sleep(0.1)
    t.progress_current = 100
    assert t.progress_last == 50
    assert t.left_time < 0.1
    assert t.progress_description in t.progress_description_with_value
    assert t.progress_description == '100.00%'



from sgtpyutils.network.TcpClient import start_client
import json
import socket
import random
import time
from sgtpyutils.logger import logger
from sgtpyutils.network.SimpleCaculateService import SimpleCaculateService


def test_service_start():
    s = SimpleCaculateService()
    s.start()
    assert s.is_listening == True
    return s


def test_caculator():
    s = test_service_start()
    client = start_client(s.port)

    rnd = random.Random()
    for i in range(10):
        a = rnd.randint(0, int(1e5))
        b = rnd.randint(0, int(1e5))
        client.send(f'{a}+{b}'.encode())
        data = client.recv(1024).decode().strip()
        data = int(data)
        c = a + b
        print(f'get result for {a} + {b} = {c} , recv {data}')
        assert c == data


def test_multi_lines():
    s = test_service_start()
    client = start_client(s.port)
    client.send(b'1+1\n')
    data = client.recv(1024)
    assert data == b'2\n', 'single line content should return direct'
    client.send(b'''
c = 1 + 1
    ''')
    data = client.recv(1024)
    result = data.decode().strip()
    result = json.loads(result)
    assert 'c' in result and result['c'] == 2


def test_exception():
    s = test_service_start()
    client = start_client(s.port)
    client.send(b'print(invalid\n')
    time.sleep(0.2)
    client.send(b'print(invalid\n')
    time.sleep(0.2)
    client.send(b'print(invalid\n')
    time.sleep(0.2)
    data = client.recv(1024)
    result = data.decode().strip()
    return test_caculator()



from sgtpyutils.hash import *

raw_md5 = 'e10adc3949ba59abbe56e057f20f883e'
raw_sha256 = '8d969eef6ecad3c29a3a629280e686cf0c3f5d5a86aff3ca12020c923adc6c92'
raw = '123456'


def test_hash():
    assert get_hash(raw, HashAlgo.md5) == raw_md5
    assert get_hash(raw, HashAlgo.sha256) == raw_sha256


def test_hash_crash():
    assert brute_hash(
      target=raw_sha256, 
      hash_name=HashAlgo.sha256,
      brute_length=6,
      table='1234567890'
    ) == raw
